/**
 * Copyright (C) 2007-2011 Linutop SAS <http://www.linutop.com/>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>

#include "utils.h"
#include "main.h"
#include "wk-browser.h"

#define DEFAULT_ITEM_TIMEOUT 5

WkSlideshowItemType
wk_slideshow_item_guess_type (const gchar *uri)
{
    gchar *type;
    gchar *filename;
    gchar *type_str;
    gchar *ext;
    gboolean is_local;
    WkSlideshowItemType ret = WK_SLIDESHOW_ITEM_TYPE_WEB;
    WkSettings *settings = GET_SETTINGS ();

    is_local = g_str_has_prefix (uri, "file://");
    filename = g_path_get_basename (uri);
    type = g_content_type_guess (uri+7, NULL, 0, NULL);

    /* lookup in the predefined extensions */
    ext = g_strrstr (filename, ".");
    if (settings->slideshow_types && ext)
    {
        ext++; /* move behind the . */
        type_str = g_hash_table_lookup (settings->slideshow_types, ext);

        if (type_str)
        {
            if (0 == strcmp ("image", type_str))
            {
                DEBUG ("type forced by config: image");
                ret = WK_SLIDESHOW_ITEM_TYPE_IMAGE;
                goto out;
            }
            else if (0 == strcmp ("video", type_str))
            {
                DEBUG ("type forced by config: video");
                ret = WK_SLIDESHOW_ITEM_TYPE_VIDEO;
                goto out;
            }
            else if (0 == strcmp ("document", type_str))
            {
                DEBUG ("type forced by config: document");
                ret = WK_SLIDESHOW_ITEM_TYPE_DOCUMENT;
                goto out;
            }
        }
        DEBUG ("type not found in config");
    }

    if (g_str_has_prefix (type, "video/")
        || g_str_has_prefix (type, "audio/"))
    {
        ret = WK_SLIDESHOW_ITEM_TYPE_VIDEO;
        goto out;
    }

    if (is_local)
    {
        if (g_str_has_prefix (type, "image/"))
        {
            ret = WK_SLIDESHOW_ITEM_TYPE_IMAGE;
            goto out;
        }

        if ((strcmp ("text/html", type) == 0) ||
            (strcmp ("text/plain", type) == 0))
        {
            ret = WK_SLIDESHOW_ITEM_TYPE_DOCUMENT;
            goto out;
        }
    }

out:
    g_free (filename);
    g_free (type);

    DEBUG("type: %d", ret);

    return ret;
}

WkSlideshowItem *
wk_slideshow_item_new (gint id,
                       const char *uri,
                       WkSlideshowItemType type,
                       int timeout,
                       gboolean href)
{
    WkSlideshowItem *item;

    item = g_new (WkSlideshowItem, 1);

    item->id = id;
    item->uri = g_strdup (uri);
    item->timeout = timeout;
    item->type = type ? type : wk_slideshow_item_guess_type (uri);
    item->href = href;

    return item;
}

WkSlideshowItem *
wk_slideshow_item_new_from_xml_data (gint id,
                                     const gchar **attribute_names,
                                     const gchar **attribute_values)
{
    int i;
    const gchar *uri = NULL;
    WkSlideshowItem *item;
    WkSlideshowItemType type = WK_SLIDESHOW_ITEM_TYPE_UNKNOWN;
    int t = 0;
    gboolean href = FALSE;
    gchar *absolute_uri;

    for (i=0; attribute_names[i] != NULL; i++)
    {
        if (0 == strcmp (attribute_names[i], "uri"))
            uri = attribute_values[i];
        else if (0 == strcmp (attribute_names[i], "time"))
            t = atoi (attribute_values[i]);
        else if (0 == strcmp (attribute_names[i], "href"))
            href = (strcmp (attribute_values[i], "1") == 0) ? TRUE : FALSE;
        else if (0 == strcmp (attribute_names[i], "type"))
        {
            if (strcmp (attribute_values[i], "image") == 0)
                type = WK_SLIDESHOW_ITEM_TYPE_IMAGE;
            else if (strcmp (attribute_values[i], "video") == 0)
                type = WK_SLIDESHOW_ITEM_TYPE_VIDEO;
            else if (strcmp (attribute_values[i], "document") == 0)
                type = WK_SLIDESHOW_ITEM_TYPE_DOCUMENT;
            else
                type = WK_SLIDESHOW_ITEM_TYPE_WEB;
        }
    }

    if (!uri)
        return NULL;

    if (type == WK_SLIDESHOW_ITEM_TYPE_UNKNOWN)
        type = wk_slideshow_item_guess_type (uri);

    /* videos can have a time value of 0 (play until the end) */
    if (!t && (type != WK_SLIDESHOW_ITEM_TYPE_VIDEO))
    {
        t = DEFAULT_ITEM_TIMEOUT;
    }

    WkSettings *settings = GET_SETTINGS ();
    if (uri[0] == '/')
        absolute_uri = g_strconcat (settings->slideshow_file,
                                    "/r", uri, NULL);
    else
        absolute_uri = g_strdup (uri);

    item = wk_slideshow_item_new (id, absolute_uri, type, t, href);

    g_free (absolute_uri);

    return item;
}

WkSlideshowItem *
wk_slideshow_item_new_idle ()
{
    return wk_slideshow_item_new (0,
                                  "idle",
                                  WK_SLIDESHOW_ITEM_TYPE_IDLE,
                                  60,
                                  FALSE);
}

void
wk_slideshow_item_free (WkSlideshowItem *item)
{
    g_free (item->uri);
    g_free (item);
}
