/**
 * Copyright (C) 2007-2011 Linutop SAS <http://www.linutop.com/>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#include <glib/gstdio.h>

#include "utils.h"
#include "wk-key-file.h"

struct _WkKeyFilePrivate
{
    GKeyFile *key_file;
    gchar *basedir;
    gchar *filenames[WK_KEY_FILE_FILENAME_COUNT];
};

static void wk_key_file_class_init (WkKeyFileClass *klass);
static void wk_key_file_init (WkKeyFile *key_file);
static void wk_key_file_finalize (GObject *object);

static void wk_key_file_build_filenames (WkKeyFile *kf, gboolean init);

G_DEFINE_TYPE (WkKeyFile, wk_key_file, G_TYPE_OBJECT);

static void
wk_key_file_class_init (WkKeyFileClass *klass)
{
    g_type_class_add_private (klass, sizeof (WkKeyFilePrivate));

    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
    gobject_class->finalize = wk_key_file_finalize;
}

static void
wk_key_file_init (WkKeyFile *key_file)
{
    key_file->priv = G_TYPE_INSTANCE_GET_PRIVATE (key_file,
                                                  WK_TYPE_KEY_FILE,
                                                  WkKeyFilePrivate);

    key_file->priv->key_file = g_key_file_new ();
    wk_key_file_build_filenames (key_file, TRUE);
}

static void
wk_key_file_finalize (GObject *object)
{
    int i;

    WkKeyFile *key_file = WK_KEY_FILE (object);

    for (i=0;i<WK_KEY_FILE_FILENAME_COUNT;i++)
        g_free (key_file->priv->filenames[i]);

    g_free (key_file->priv->basedir);
    g_key_file_free (key_file->priv->key_file);

    (*G_OBJECT_CLASS (wk_key_file_parent_class)->finalize) (object);
}

/*
 * utils
 */
static gint
_key_file_get_int (GKeyFile *file,
                   const gchar *section,
                   const gchar *key,
                   gint fallback)
{
    GError *error = NULL;
    gint ret;

    if (!(ret = g_key_file_get_integer (file, section, key, &error))
        && (error != NULL))
    {
        ret = fallback;
        g_error_free (error);
        error = NULL;
    }

    return ret;
}

static gboolean
_key_file_get_boolean (GKeyFile *file,
                       const gchar *section,
                       const gchar *key,
                       gboolean fallback)
{
    GError *error = NULL;
    gboolean ret;

    if (!(ret = g_key_file_get_boolean (file, section, key, &error))
        && (error != NULL))
    {
        ret = fallback;
        g_error_free (error);
        error = NULL;
    }

    return ret;
}

static gchar *
_key_file_get_string (GKeyFile *file,
                      const gchar *section,
                      const gchar *key,
                      const gchar *fallback)
{
    gchar *ret = NULL;

    ret = g_key_file_get_string (file, section, key, NULL);
    if (!ret || g_strcmp0 (ret, "") == 0)
    {
        g_free (ret);
        if (fallback != NULL)
            return g_strdup (fallback);
        else
            return NULL;
    }

    return ret;
}

static void
wk_key_file_build_filenames (WkKeyFile *kf, gboolean init)
{
    int i;

    if (init)
    {
        for (i=0;i<WK_KEY_FILE_FILENAME_COUNT;i++)
            kf->priv->filenames[i] = NULL;
        return;
    }

    for (i=0;i<WK_KEY_FILE_FILENAME_COUNT;i++)
    {
        g_free (kf->priv->filenames[i]);

        switch (i)
        {
            case WK_KEY_FILE_FILENAME_PREFERENCES:
                kf->priv->filenames[i] =
                    wk_key_file_build_filename (kf, "preferences");
                break;
            case WK_KEY_FILE_FILENAME_PLAYLIST:
                kf->priv->filenames[i] =
                    wk_key_file_build_filename (kf, "playlist");
                break;
            case WK_KEY_FILE_FILENAME_PRESTART:
                kf->priv->filenames[i] =
                    wk_key_file_build_filename (kf, "prestart");
                break;
            case WK_KEY_FILE_FILENAME_LIST:
                kf->priv->filenames[i] =
                    wk_key_file_build_filename (kf, "list");
                break;
            case WK_KEY_FILE_FILENAME_HOOKS:
                kf->priv->filenames[i] =
                    wk_key_file_build_filename (kf, "hooks");
                break;
            case WK_KEY_FILE_FILENAME_COOKIES:
                kf->priv->filenames[i] =
                    wk_key_file_build_filename (kf, "cookies.sqlite");
                break;
        }
    }
}

/*
 * Public API
 */
WkKeyFile *
wk_key_file_new (gchar *basedir)
{
    WkKeyFile *kf = WK_KEY_FILE (g_object_new (WK_TYPE_KEY_FILE, NULL));
    kf->priv->basedir = g_strdup (basedir ? basedir
                                          : WK_DEFAULT_CONFDIR);
    wk_key_file_build_filenames (kf, FALSE);

    return kf;
}

inline const gchar *
wk_key_file_get_basedir (WkKeyFile *key_file)
{
    return key_file->priv->basedir;
}

inline void
wk_key_file_set_basedir (WkKeyFile *key_file, gchar *basedir)
{
    g_free (key_file->priv->basedir);
    key_file->priv->basedir = g_strdup (basedir);
    wk_key_file_build_filenames (key_file, FALSE);
}

inline const gchar *
wk_key_file_get_filename (WkKeyFile *key_file,
                          WkKeyFileFilename name)
{
    return key_file->priv->filenames[name];
}

inline gchar *
wk_key_file_build_filename (WkKeyFile *key_file, const gchar *name)
{
    return g_build_filename (key_file->priv->basedir, name, NULL);
}

gboolean
wk_key_file_load_from_data (WkKeyFile *key_file, const gchar *data)
{
    g_return_val_if_fail (WK_IS_KEY_FILE (key_file), FALSE);
    g_return_val_if_fail (data != NULL, FALSE);

    return g_key_file_load_from_data (key_file->priv->key_file,
                                      data,
                                      g_utf8_strlen (data, -1),
                                      G_KEY_FILE_KEEP_COMMENTS,
                                      NULL);
}

gboolean
wk_key_file_load_from_file (WkKeyFile *key_file, const gchar *filename)
{
    gboolean ret;
    gchar *tmp;

    g_return_val_if_fail (WK_IS_KEY_FILE (key_file), FALSE);
    g_return_val_if_fail (filename != NULL, FALSE);

    if (g_path_is_absolute (filename))
        tmp = g_strdup (filename);
    else
        tmp = g_build_filename (key_file->priv->basedir, filename, NULL);

    ret = g_key_file_load_from_file (key_file->priv->key_file,
                                     tmp,
                                     G_KEY_FILE_KEEP_COMMENTS,
                                     NULL);
    g_free (tmp);

    return ret;
}

gboolean
wk_key_file_load (WkKeyFile *key_file)
{
    return wk_key_file_load_from_file (key_file, "preferences");
}

gchar **
wk_key_file_get_keys (WkKeyFile   *key_file,
                      const gchar *section,
                      gsize       *length,
                      GError     **error)
{
    g_return_val_if_fail (WK_IS_KEY_FILE (key_file), NULL);

    return g_key_file_get_keys (key_file->priv->key_file,
                                section, length, error);
}

gchar **
wk_key_file_get_sections (WkKeyFile *key_file,
                          gsize     *length)
{
    g_return_val_if_fail (WK_IS_KEY_FILE (key_file), NULL);

    return g_key_file_get_groups (key_file->priv->key_file, length);
}

WkConfigItem *
wk_key_file_get_config_item (WkKeyFile *key_file,
                             const gchar *section,
                             const gchar *key,
                             WkConfigItemType type,
                             gpointer fallback)
{
    WkConfigItem *item;
    gpointer value = NULL;
    gint int_value;
    gboolean bool_value;
    gchar *str_value;
    gchar *uri = NULL;

    switch (type)
    {
    case WK_CONFIG_ITEM_TYPE_INT:
        int_value = _key_file_get_int (key_file->priv->key_file,
                                       section,
                                       key,
                                       GPOINTER_TO_INT (fallback));
        value = GINT_TO_POINTER (int_value);
        break;

    case WK_CONFIG_ITEM_TYPE_BOOLEAN:
        bool_value = _key_file_get_boolean (key_file->priv->key_file,
                                        section,
                                        key,
                                        (gboolean) GPOINTER_TO_INT (fallback));
        value = GINT_TO_POINTER ((gint) bool_value);
        break;

     case WK_CONFIG_ITEM_TYPE_STRING:
        str_value = _key_file_get_string (key_file->priv->key_file,
                                          section,
                                          key,
                                          (const gchar *) fallback);
        value = str_value;
        break;

    default:
       break;
    }

    item = wk_config_item_new (section, key, uri, type, value);
    g_free (uri);

    return item;
}

void
wk_key_file_set_config_item (WkKeyFile *key_file,
                             WkConfigItem *item)
{
    GKeyFile *gkf = key_file->priv->key_file;
    const gchar *section = item->section;
    const gchar *key = item->key;

    switch (item->type)
    {
    case WK_CONFIG_ITEM_TYPE_INT:
        g_key_file_set_integer (gkf, section, key,
                                GPOINTER_TO_INT (item->value));
        break;

    case WK_CONFIG_ITEM_TYPE_BOOLEAN:
        g_key_file_set_boolean (gkf, section, key,
                                (gboolean) GPOINTER_TO_INT (item->value));
        break;

    case WK_CONFIG_ITEM_TYPE_STRING:
        g_key_file_set_string (gkf, section, key,
                                (gchar *) item->value);
        break;

    default:
        break;
    }
}

gint
wk_key_file_get_int (WkKeyFile   *key_file,
                     const gchar *section,
                     const gchar *key,
                     int          fallback)
{
    g_return_val_if_fail (WK_IS_KEY_FILE (key_file), fallback);

    gint ret;
    WkConfigItem *item;

    item = wk_key_file_get_config_item (key_file, section, key,
                                        WK_CONFIG_ITEM_TYPE_INT,
                                        GINT_TO_POINTER (fallback));

    if (!item)
        return fallback;

    ret = GPOINTER_TO_INT (wk_config_item_get_value (item));
    wk_config_item_free (item);

    return ret;
}

gboolean
wk_key_file_get_boolean (WkKeyFile   *key_file,
                         const gchar *section,
                         const gchar *key,
                         gboolean     fallback)
{
    g_return_val_if_fail (WK_IS_KEY_FILE (key_file), fallback);

    gboolean ret;
    WkConfigItem *item;

    item = wk_key_file_get_config_item (key_file, section, key,
                                        WK_CONFIG_ITEM_TYPE_BOOLEAN,
                                        GINT_TO_POINTER ((gint) fallback));
    if (!item)
        return fallback;

    ret = (gboolean) GPOINTER_TO_INT (wk_config_item_get_value (item));
    wk_config_item_free (item);

    return ret;
}

gchar *
wk_key_file_get_string (WkKeyFile  *key_file,
                        const gchar *section,
                        const gchar *key,
                        const gchar *fallback)
{
    g_return_val_if_fail (WK_IS_KEY_FILE (key_file), g_strdup (fallback));

    gchar *ret;
    WkConfigItem *item;

    item = wk_key_file_get_config_item (key_file, section, key,
                                        WK_CONFIG_ITEM_TYPE_STRING,
                                        (gpointer) fallback);
    if (!item)
        return g_strdup (fallback);

    ret = g_strdup ((gchar *) (wk_config_item_get_value (item)));
    wk_config_item_free (item);

    return ret;
}

gchar *
wk_key_file_get_data (WkKeyFile *key_file)
{
    GString *str;
    gchar **sections = NULL;
    gchar **sectionp = NULL;
    gchar **keys = NULL;
    gchar **keyp = NULL;
    gchar *value = NULL;
    GKeyFile *gkf = key_file->priv->key_file;

    str = g_string_new ("");

    sections = g_key_file_get_groups (gkf, NULL);

    for (sectionp = sections; *sectionp; sectionp++)
    {
        gchar *section = *sectionp;
        g_string_append_printf (str, "[%s]\n", section);

        keys = g_key_file_get_keys (gkf, section, NULL, NULL);

        for (keyp = keys; *keyp; keyp++)
        {
            gchar *key = *keyp;
            value = g_key_file_get_value (gkf, section, key, NULL);
            g_string_append_printf (str, "%s = %s\n", key, value);
            g_free (value);
            value = NULL;
        }
        g_string_append (str, "\n");
        g_strfreev (keys);
        keys = NULL;
    }
    g_strfreev (sections);

    return g_string_free (str, FALSE);
}

gboolean
wk_key_file_write (WkKeyFile *key_file, const gchar *filename)
{
    gchar *data;
    gboolean ret = FALSE;

    data = wk_key_file_get_data (key_file);
    if (data)
    {
        if (! create_dir_for_file (filename))
        {
            ret = FALSE;
            goto out;
        }

        ret = g_file_set_contents (filename, data, -1, NULL);
    }

out:
    g_free (data);
    return ret;
}
